#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The apl legacy fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type


import platform
import re

from ansible_collections.apresia.apl.plugins.module_utils.network.apl.apl import (
    run_commands,
    get_capabilities,
)


class FactsBase(object):

    COMMANDS = list()

    def __init__(self, module):
        self.module = module
        self.facts = dict()
        self.warnings = list()
        self.responses = None

    def populate(self):
        self.responses = run_commands(
            self.module, commands=self.COMMANDS, check_rc=False
        )

    def run(self, cmd):
        return run_commands(self.module, commands=cmd, check_rc=False)


class Default(FactsBase):

    COMMANDS = ["show unit"]

    def populate(self):
        super(Default, self).populate()
        self.facts.update(self.platform_facts())
        data = self.responses[0]
        if data:
            self.facts["serialnum"] = self.parse_serialnum(data)
        
    def parse_serialnum(self, data):
        np_show_unit_regexp = r"""Unit\s+Serial-Number\s+Status\s+Up Time\s?
[- ]+
\s+(\d+)\s+(?P<serial_number>\S+)"""
        if show_unit := re.search(np_show_unit_regexp, data):
            return show_unit["serial_number"]

    def platform_facts(self):
        platform_facts = {}

        resp = get_capabilities(self.module)
        device_info = resp["device_info"]

        platform_facts["system"] = device_info["network_os"]

        for item in ("model", "version"):
            val = device_info.get("network_os_%s" % item)
            if val:
                platform_facts[item] = val

        platform_facts["api"] = resp["network_api"]
        platform_facts["python_version"] = platform.python_version()

        return platform_facts


class Hardware(FactsBase):

    COMMANDS = ["dir", "show unit"]

    def populate(self):
        super(Hardware, self).populate()
        data = self.responses[0]
        if data:
            self.facts["filesystems"] = self.parse_filesystems(data)
            self.facts["filesystems_info"] = self.parse_filesystems_info(data)

        data = self.responses[1]
        meminfo = self.parse_memory_info(data)
        if meminfo:
            self.facts["memtotal_mb"] = int(meminfo[0]) / 1024
            self.facts["memfree_mb"] = int(meminfo[1]) / 1024

    def parse_filesystems(self, data):
        return re.findall(r"^Directory of (\S+)", data, re.M)

    def parse_filesystems_info(self, data):
        facts = dict()
        fs = ""
        for line in data.split("\n"):
            match = re.match(r"^Directory of (\S+)", line)
            if match:
                fs = match.group(1)
                facts[fs] = dict()
                continue
            match = re.match(r"^(\d+) bytes total \((\d+) bytes free\)", line)
            if match:
                facts[fs]["spacetotal_kb"] = int(match.group(1)) / 1024
                facts[fs]["spacefree_kb"] = int(match.group(2)) / 1024
        return facts

    def parse_memory_info(self, data):
        memory_regexp = r"DRAM\s+(?P<mem_total>\d+)\s+K\s+\d+\s+K\s+(?P<mem_free>\d+)\s+K"
        if meminfo := re.search(memory_regexp, data):
            return (meminfo["mem_total"], meminfo["mem_free"])

class Config(FactsBase):

    COMMANDS = ["show running-config"]

    def populate(self):
        super(Config, self).populate()
        data = self.responses[0]
        if data:
            data = re.sub(
                r"^Building configuration...\s+Current configuration : \d+ bytes\n",
                "",
                data,
                flags=re.MULTILINE,
            )
            self.facts["config"] = data


class Interfaces(FactsBase):

    COMMANDS = [
        "show interfaces",
        "show ip interface",
        "show ipv6 interface",
    ]

    def populate(self):
        super(Interfaces, self).populate()

        self.facts["all_ipv4_addresses"] = list()
        self.facts["all_ipv6_addresses"] = list()

        data = self.responses[0]
        if data:
            self.facts["interfaces"] = self.populate_interfaces(data)

        data = self.responses[1]
        if data:
            self.populate_ipv4_interfaces(data)

        data = self.responses[2]
        if data:
            self.populate_ipv6_interfaces(data)

    def populate_interfaces(self, data):
        facts = dict()
        interface_regexp = r"^(\S+).+?link status is (\S+).+?(?=^$)"
        interfaces = re.compile(interface_regexp, re.DOTALL | re.M)
        for interface in interfaces.finditer(data):
            intf = dict()
            value = interface.group(0)
            key = interface.group(1)
            line_protocol = interface.group(2)

            intf["description"] = self.parse_description(value)
            intf["macaddress"] = self.parse_macaddress(value)
            intf["mtu"] = self.parse_mtu(value)
            intf["mediatype"] = self.parse_mediatype(value)
            intf["duplex"] = self.parse_duplex(value)
            intf["lineprotocol"] = line_protocol

            facts[key] = intf

        return facts

    def populate_ipv4_interfaces(self, data):
        ip_interface_regexp = r"^Interface (\S+).+?link status is (\S+).+?(?=^$)"
        interfaces = re.compile(ip_interface_regexp, re.DOTALL | re.M)
        for interface in interfaces.finditer(data):
            informations = interface.group(0)
            if ip_address := re.search(r"IP address is (\S+)", informations):
                self.add_ip_address(ip_address.group(1), "ipv4")

    def populate_ipv6_interfaces(self, data):
        ipv6_interface_regexp = r"^(\S+).+?link status is (\S+).+?(?=^$)"
        interfaces = re.compile(ipv6_interface_regexp, re.DOTALL | re.M)
        for interface in interfaces.finditer(data):
            informations = interface.group(0)
            if ip_address := re.search(r"Global unicast address:.+?^\s+(\S+)", informations, re.DOTALL | re.MULTILINE):
                self.add_ip_address(ip_address.group(1), "ipv6")

    def add_ip_address(self, address, family):
        if family == "ipv4":
            self.facts["all_ipv4_addresses"].append(address)
        else:
            self.facts["all_ipv6_addresses"].append(address)

    def parse_description(self, data):
        match = re.search(r"Interface description: (\S+)", data)
        if match:
            return match.group(1)

    def parse_macaddress(self, data):
        match = re.search(r"MAC Address: (\S+)", data)
        if match:
            return match.group(1)

    def parse_ipv4(self, data):
        match = re.search(r"Internet address is (\S+)", data)
        if match:
            addr, masklen = match.group(1).split("/")
            return dict(address=addr, masklen=int(masklen))

    def parse_mtu(self, data):
        match = re.search(r"Maximum transmit unit:\s+(\d+)\s+bytes", data)
        if match:
            return int(match.group(1))

    def parse_duplex(self, data):
        match = re.search(r"(\S+)-duplex", data)
        if match:
            return match.group(1)

    def parse_mediatype(self, data):
        match = re.search(r"Interface type: (\S+)", data)
        if match:
            return match.group(1)
